//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License
//     as published by the Free Software Foundation; either version 2.1
//     of the License, or any later version.
//
//     This library is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//     Boston, MA 02111-1307 USA
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.3 (15112), 2011/05/27 16:00:14
//                        $Revision: 14936 $
//                        $Date: 2010-11-05 15:57:40 +0100 (Fri, 05 Nov 2010) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import de.knipp.rritoolkit.Port;

import de.knipp.rritoolkit.response.AuthInfo1DeleteResponse;

/**
 *  class that represents an AuthInfo 1 delete request
 *
 *  @version  $Revision: 14936 $
 *  @author   Klaus Malorny
 */

public class AuthInfo1DeleteRequest extends DomainSimpleRequest
{


  /**
   *  constructor
   */

  public AuthInfo1DeleteRequest ()
  {
    // empty
  }


  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   *  @param theEncodedDomainName   the encoded domain name, may be {@code null}
   *                                for ASCII only domains
   */

  public AuthInfo1DeleteRequest (String theDomainName,
    String theEncodedDomainName)
  {
    super (theDomainName, theEncodedDomainName);
  }



  /**
   *  constructor
   *
   *  @param theDomainName          the domain name
   */

  public AuthInfo1DeleteRequest (String theDomainName)
  {
    super (theDomainName);
  }

  

//--- implementation of Request ----------------------------------------------



  /**
   *  execute the this request on the given port.
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */

  @Override
  public AuthInfo1DeleteResponse execute (Port port)
    throws IOException
  {
    return (AuthInfo1DeleteResponse) super.execute (port);
  }



  /**
   *  return the XML element name
   *
   *  @return                      the element name
   */

  @Override
  protected String getRequestElementName ()
  {
    return "deleteAuthInfo1";
  }





  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "delete-authinfo1";
  }



}


