//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.3 (15112), 2011/05/27 16:00:14
//                        $Revision: 14936 $ 
//                        $Date: 2010-11-05 15:57:40 +0100 (Fri, 05 Nov 2010) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.request;

import java.io.IOException;

import org.w3c.dom.Node;
import org.w3c.dom.DOMException;

import de.knipp.rritoolkit.KVList;
import de.knipp.rritoolkit.XmlBuilder;
import de.knipp.rritoolkit.ContactData;
import de.knipp.rritoolkit.Port;
import de.knipp.rritoolkit.EncodeException;

import de.knipp.rritoolkit.response.ContactUpdateResponse;

import static de.knipp.rritoolkit.DenicConstants.NS_CONTACT;
import static de.knipp.rritoolkit.DenicConstants.PREFIX_CONTACT;



/**
 *  request to create a given contact
 *
 *  @version  $Revision: 14936 $
 *  @author   Klaus Malorny
 */

public class ContactUpdateRequest extends Request
{
  /** the contact data */
  
  private ContactData contact;
  
  
  /**
   *  constructor
   */
  
  public ContactUpdateRequest ()
  {
    // empty
  }
  
  
  /**
   *  constructor
   *
   *  @param theContact             the contact
   */
  
  public ContactUpdateRequest (ContactData theContact)
  {
    setContact (theContact);
  }
  
  
  //--- implementation of Request ----------------------------------------------
  

  /**
   *  executes the this request on the given port. 
   *
   *  @param port                 the port on which this request shall
   *                              be executed
   *  @return                     the response
   *  @throws IOException         if the execution failed
   */

  @Override
  public ContactUpdateResponse execute (Port port)
    throws IOException
  {
    return (ContactUpdateResponse) super.execute (port);
  }
  
    
  /**
   *  method that needs to be implemented by subclasses for the XML conversion
   *
   *  @param node                 the node where the request shall be added to
   *  @throws DOMException        on DOM exceptions
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void requestToXml (Node node)
    throws DOMException, EncodeException
  {
    if (contact == null) throw new EncodeException ("contact not set");
    
    XmlBuilder xb = new XmlBuilder (node);
    xb.es (NS_CONTACT, PREFIX_CONTACT, "update");
    contact.toXml (xb.getCurrent ());
    xb.ee ();
  }
  

  
  
  
  /**
   *  return the action string for this request
   *
   *  @return                     the action string
   */

  @Override
  protected String getActionCode ()
  {
    return "update";
  }
  

  
  /**
   *  method that needs to be implemented by subclasses for the 
   *  key-value list conversion
   *
   *  @param list                   the list that has to be filled
   *  @throws EncodeException     if the encoding fails for some reason
   */

  @Override
  protected void requestToKVList (KVList list)
    throws EncodeException
  {
    if (contact == null) throw new EncodeException ("contact not set");
    contact.toList (list);
  }
  
  
  
  
  /**
   *  return a string representation for debugging purposes
   *
   *  @return                 the representation
   */

  @Override
  public String toString ()
  {
    return super.toString () + contact;
  }
  

  //--- access methods ---------------------------------------------------------
  
  /**
   *  set the contact that shall be created
   *
   *  @param theContact             the contact
   */
  
  public void setContact (ContactData theContact)
  {
    contact = theContact;
  }
  
  
  
  /**
   *  return the contact
   *
   *  @return                       the contact or {@code null} if it
   *                                hasn't been set yet
   */
  
  public ContactData getContact ()
  {
    return contact;
  }
  
}