//______________________________________________________________________________
//   |       |
//   | knipp |           Knipp Medien und Kommunikation GmbH
//    -------
//
//
//     Knipp RRI Toolkit
//
//
//     Copyright (C) 2005 by Knipp Medien und Kommunikation GmbH
//
//     This library is free software; you can redistribute it and/or
//     modify it under the terms of the GNU Lesser General Public License 
//     as published by the Free Software Foundation; either version 2.1 
//     of the License, or any later version.
// 
//     This library is distributed in the hope that it will be useful, 
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
//     GNU Lesser General Public License for more details.
//
//     You should have received a copy of the GNU Lesser General
//     Public License along with this library; if not, write to the 
//     Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
//     Boston, MA 02111-1307 USA 
//
//
//     initial author:    Klaus Malorny
//
//     version:           1.3 (15112), 2011/05/27 16:00:14
//                        $Revision: 14936 $ 
//                        $Date: 2010-11-05 15:57:40 +0100 (Fri, 05 Nov 2010) $
//                        $Author: Klaus.Malorny $
//

package de.knipp.rritoolkit.response;

/**
 *  abstract implementation that implements the {@code visit} method
 *  for {@code Request} as an empty method and all other methods
 *  to call this method
 *
 *  @version  $Revision: 14936 $
 *  @author   Klaus Malorny
 */

public abstract class AbstractResponseVisitor implements ResponseVisitor
{
  /**
   *  called by a generic response
   *
   *  @param response                 the response
   */

  @Override
  public void visit (Response response)
  {
    // empty
  }
  
  
  /**
   *  called by a contact check response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (ContactCheckResponse response)
  {
    visit ((Response) response);
  }
  
  
  
  /**
   *  called by a contact create response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (ContactCreateResponse response)
  {
    visit ((Response) response);
  }
  
  
  
  /**
   *  called by a contact info response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (ContactInfoResponse response)
  {
    visit ((Response) response);
  }

  
  
  /**
   *  called by a contact update response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (ContactUpdateResponse response)
  {
    visit ((Response) response);
  }
  
  
  
  /**
   *  called by a domain change holder response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainChangeHolderResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by a domain change prov response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainChangeProvResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by a domain check response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainCheckResponse response)
  {
    visit ((Response) response);
  }
  
  
  
  /**
   *  called by a domain create response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainCreateResponse response)
  {
    visit ((Response) response);
  }
  

  
  /**
   *  called by a domain delete response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainDeleteResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by a domain info response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainInfoResponse response)
  {
    visit ((Response) response);
  }
  
  
  
  /**
   *  called by a domain transit response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainTransitResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by a domain update response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainUpdateResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by a domain migrate description response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (DomainMigrateDescrResponse response)
  {
    visit ((Response) response);
  }

  
  /**
   *  called by a login response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (LoginResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by a logout response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (LogoutResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by queue delete response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (QueueDeleteResponse response)
  {
    visit ((Response) response);
  }
  
  
  /**
   *  called by a queue read response
   *
   *  @param response                 the response
   */
  
  @Override
  public void visit (QueueReadResponse response)
  {
    visit ((Response) response);
  }


  /**
   *  called by an AuthInfo 1 create response
   *
   *  @param response                 the response
   */

  @Override
  public void visit (AuthInfo1CreateResponse response)
  {
    visit ((Response) response);
  }



  /**
   *  called by an AuthInfo 1 delete response
   *
   *  @param response                 the response
   */

  @Override
  public void visit (AuthInfo1DeleteResponse response)
  {
    visit ((Response) response);
  }



  /**
   *  called by an AuthInfo 2 create response
   *
   *  @param response                 the response
   */

  @Override
  public void visit (AuthInfo2CreateResponse response)
  {
    visit ((Response) response);
  }


}



